"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useState = useState;
exports.useEffect = useEffect;
exports.useContext = useContext;
exports.useReducer = useReducer;
exports.useCallback = useCallback;
exports.useMemo = useMemo;
exports.useRef = useRef;
exports.useImperativeMethods = useImperativeMethods;
exports.useMutationEffect = useMutationEffect;
exports.useLayoutEffect = useLayoutEffect;
exports.default = withHooks;

var _react = _interopRequireDefault(require("react"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } return _assertThisInitialized(self); }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var currentInstance = null;
var isMounting = false;
var callIndex = 0;

function useState(initial) {
  var id = ++callIndex;
  var state = currentInstance.state;
  var setState = currentInstance.setState.bind(currentInstance);

  var updater = function updater(newValue) {
    setState(_defineProperty({}, id, newValue));
  };

  if (isMounting) {
    state[id] = typeof initial === 'function' ? initial() : initial;
    return [state[id], updater];
  } else {
    return [state[id], updater];
  }
}

function useEffect(rawEffect, deps) {
  var id = ++callIndex;

  if (isMounting) {
    var injectedCleanup = function injectedCleanup() {
      var current = injectedCleanup.current;

      if (current) {
        current();
        injectedCleanup.current = null;
      }
    };

    var injectedEffect = function injectedEffect() {
      injectedCleanup();
      var current = injectedEffect.current;

      if (current) {
        injectedCleanup.current = current();
      }
    };

    injectedEffect.current = rawEffect;
    currentInstance._hookStore[id] = {
      effect: injectedEffect,
      cleanup: injectedCleanup,
      deps: deps
    };
    injectEffect('componentDidMount', injectedEffect);
    injectEffect('componentWillUnmount', injectedCleanup);
    injectEffect('componentDidUpdate', injectedEffect);
    injectEffect('componentWillUpdate', injectedCleanup);
  } else {
    var _currentInstance$_hoo = currentInstance._hookStore[id],
        effect = _currentInstance$_hoo.effect,
        _currentInstance$_hoo2 = _currentInstance$_hoo.deps,
        prevDeps = _currentInstance$_hoo2 === void 0 ? [] : _currentInstance$_hoo2;

    if (!deps || deps.some(function (d, i) {
      return d !== prevDeps[i];
    })) {
      effect.current = rawEffect;
    } else {
      effect.current = null;
    }
  }
}

function useContext(Context) {
  if (isMounting) {
    var originalRender = currentInstance.render.bind(currentInstance);

    currentInstance.render = function () {
      return _react.default.createElement(Context.Consumer, null, originalRender);
    };
  }

  return Context._currentValue;
}

function useReducer(reducer, initialState, initialAction) {
  if (isMounting && initialAction) {
    initialState = reducer(initialState, initialAction);
  }

  var _useState = useState(initialState),
      _useState2 = _slicedToArray(_useState, 2),
      state = _useState2[0],
      setState = _useState2[1];

  function dispatch(action) {
    setState(reducer(state, action));
  }

  return [state, dispatch];
}

function injectEffect(key, fn) {
  currentInstance[key] = fn;
}

function useCallback(fn, deps) {
  return useMemo(function () {
    return fn;
  }, deps);
}

function useMemo(fn, deps) {
  var id = ++callIndex;

  if (isMounting) {
    var result = fn();
    currentInstance._hookStore[id] = {
      deps: deps,
      result: result
    };
    return result;
  } else {
    var _currentInstance$_hoo3 = currentInstance._hookStore[id],
        _result = _currentInstance$_hoo3.result,
        _currentInstance$_hoo4 = _currentInstance$_hoo3.deps,
        prevDeps = _currentInstance$_hoo4 === void 0 ? [] : _currentInstance$_hoo4;

    if (!deps || deps.some(function (d, i) {
      return d !== prevDeps[i];
    })) {
      var _result2 = fn();

      currentInstance._hookStore[id] = {
        deps: deps,
        result: _result2
      };
      return _result2;
    } else {
      return _result;
    }
  }
}

function useRef(initial) {
  var id = ++callIndex;

  if (isMounting) {
    var ref = {
      current: initial
    };
    currentInstance._hookStore[id] = ref;
    return ref;
  } else {
    return currentInstance._hookStore[id];
  }
}

function useImperativeMethods(ref, createInstance, deps) {
  var instance = useMemo(createInstance, deps);
  ref.current = instance;
}

function useMutationEffect() {
  return useEffect.apply(void 0, arguments);
}

function useLayoutEffect() {
  return useEffect.apply(void 0, arguments);
}

function withHooks(_render) {
  var WithHooks =
  /*#__PURE__*/
  function (_React$Component) {
    _inherits(WithHooks, _React$Component);

    function WithHooks(props) {
      var _this;

      _classCallCheck(this, WithHooks);

      _this = _possibleConstructorReturn(this, _getPrototypeOf(WithHooks).call(this));
      _this.state = {};
      _this._hookStore = [];
      currentInstance = _assertThisInitialized(_assertThisInitialized(_this));
      isMounting = true;
      _this.ret = _render(props, props._forwardedRef);
      return _this;
    }

    _createClass(WithHooks, [{
      key: "render",
      value: function render() {
        if (isMounting) {
          isMounting = false;
          callIndex = 0;
          return this.ret;
        }

        currentInstance = this;

        var ret = _render(this.props, this.props._forwardedRef);

        currentInstance = null;
        callIndex = 0;
        return ret;
      }
    }]);

    return WithHooks;
  }(_react.default.Component);

  WithHooks.displayName = _render.displayName || _render.name;

  var wrap = function wrap(props, ref) {
    return _react.default.createElement(WithHooks, _extends({}, props, {
      _forwardedRef: ref
    }));
  };

  wrap.displayName = "WithHooks(".concat(WithHooks.displayName, ")");
  return wrap;
}